"
 COPYRIGHT (c) 1993 by Claus Gittinger
 COPYRIGHT (c) 2016 Jan Vrany
 COPYRIGHT (c) 2021 LabWare
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:libview' }"

"{ NameSpace: Smalltalk }"

ResourcePack subclass:#ViewStyle
	instanceVariableNames:'name is3D viewGrey'
	classVariableNames:''
	poolDictionaries:''
	category:'Views-Support'
!

!ViewStyle class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 1993 by Claus Gittinger
 COPYRIGHT (c) 2016 Jan Vrany
 COPYRIGHT (c) 2021 LabWare
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    instances of this class keep all view-style specific information.
    The current viewStyle is kept in Views-classvariable called 'StyleSheet'
    and is instantiated with 'View defaultStyle:aStyleSymbol', which reads
    a stylesheet from a file '<aStyleSymbol>.style' (usually in the 'resources'
    directory.

    [see also:]
	View
	( Configuration & Customization :html: custom/TOP.html#VIEWSTYLE )

    [author:]
	Claus Gittinger
"
! !

!ViewStyle class methodsFor:'instance creation'!

for: class 
    ^ self for: class cached: true

    "
    ViewStyle for: SystemBrowser
    "

    "Created: / 21-09-2021 / 14:53:35 / Jan Vrany <jan.vrany@labware.com>"
!

for: class cached: cached
    ^ self forPackage: class package cached: cached

    "Created: / 21-09-2021 / 14:52:45 / Jan Vrany <jan.vrany@labware.com>"
!

forPackage:package cached:cached
    "Lookup stylesheet for given `package`. If `cached` is `true` then resulting
     stylesheet is cached to avoid reading again and again.

    The default stylesheet is looked up in package's 'styles' directory
    as follows: 
        1. look for style file with the same name as current view's style,
           for example, if current view style is 'Adwaita.style' then look
           for file named 'Adwaita.style'.
        2. look for style named either  'light.style' (for light themes, the
           default) or 'dark.style' for (dark themes), depending on current 
           style variant
        3. look for style name 'default.style'. Note, that 'default.style' may
           still contain conditional definitions based on theme variant. 
        4. if none of the above is found, the current stylesheet is returned
           (that is, `SimpleView styleSheet` - this is to avoid excessive stylesheet
           creation for packages that do not have styles).

        The current stylesheet (`SimpleView styleSheet`) is set as a super-stylesheet
        This is done prior reading any individual style files so style files can use 
        abstract definitios (like 'viewBackground') from current style. 


    "

    |styleFile style stylesDir|

    "/ Special hack for stx:libview
    package = 'stx:libview' ifTrue: [ ^ SimpleView styleSheet ].

    stylesDir := (Smalltalk projectDirectoryForPackage:package) / 'styles'.
    styleFile := stylesDir / (SimpleView styleSheet packsFileName asFilename baseName).
    cached ifTrue:[
        style := self searchCacheFor:styleFile pathName.
        style notNil ifTrue:[^ style].
    ].
    styleFile exists ifFalse: [
        styleFile := stylesDir / ((SimpleView styleSheet at:'variant') = 'dark' ifTrue:['dark.style'] ifFalse:[ 'light.style' ]).
        cached ifTrue:[
            style := self searchCacheFor:styleFile pathName.
            style notNil ifTrue:[^ style].
        ].
        styleFile exists ifFalse: [
            styleFile := stylesDir / 'default.style'.
            cached ifTrue:[
                style := self searchCacheFor:styleFile pathName.
                style notNil ifTrue:[^ style].
            ].
        ]
    ].

    styleFile exists ifTrue: [ 
        | failedToRead |

        style := ViewStyle new.
        style superPack: SimpleView styleSheet.        

        failedToRead := (style readFromFile: styleFile baseName directory: styleFile directory) isNil.

        style name: styleFile withoutSuffix baseName.         
        style packsClassOrFileName: styleFile pathName.
        style at:#fileReadFailed put:failedToRead.    
        cached ifTrue: [
            self addToCache: style.
        ].
    ] ifFalse: [ 
        style := SimpleView styleSheet.
    ].

    ^ style

    "
    ViewStyle forPackage:'stx:libview' cached:false
    ViewStyle forPackage:'stx:libtool' cached:false
    (ViewStyle forPackage:'stx:libtool' cached:true)
    (ViewStyle forPackage:'stx:libtool' cached:true) == (ViewStyle forPackage:'stx:libtool' cached:true)
    "

    "Created: / 21-09-2021 / 14:03:36 / Jan Vrany <jan.vrany@labware.com>"
    "Modified (comment): / 21-09-2021 / 15:03:40 / Jan Vrany <jan.vrany@labware.com>"
!

forPackage:package resourceFileName:resourceFileName cached:cached
    self shouldNotImplement
!

fromFile:aFileNameArg
    "get the preferences definitions from a file"

    |aFileName prefs failed nm nmInStyles dir path baseName|

    aFileName := aFileNameArg asFilename.
    (aFileName hasSuffix:'style') ifTrue:[
        baseName := aFileName withoutSuffix baseName.
        nm := aFileName.
    ] ifFalse:[
        baseName := aFileName baseName.
        nm := aFileName withSuffix:'style'.
    ].
    nm isAbsolute ifFalse:[
        nmInStyles := 'styles' asFilename / nm
    ].

    prefs := self new.
    prefs at:#name put:baseName. 

    nmInStyles notNil ifTrue:[
        path := Smalltalk getResourceFileName:nmInStyles forPackage:'stx:libview'.
    ].
    path isNil ifTrue:[
        path := Smalltalk getResourceFileName:nm forPackage:'stx:libview'.
    ].
    path notNil ifTrue:[
        dir := path asFilename directory pathName.
    ].
    failed := (prefs readFromFile:nm directory:dir) isNil.
    prefs at:#fileReadFailed put:failed. 
    ^ prefs

    "
     ViewStyle fromFile:'motif'  
     ViewStyle fromFile:'motif.style'  
     ViewStyle fromFile:'normal.style' 
     ViewStyle fromFile:'iris.style'
     ViewStyle fromFile:'iris.style' asFilename
    "

    "Modified: / 10.12.1999 / 16:05:30 / cg"
! !

!ViewStyle class methodsFor:'constants'!

adwaita
    ^ #Adwaita

    "Created: / 24-11-2016 / 17:54:58 / cg"
!

macosx_yosemite
    ^ #'macosx_yosemite'

    "Created: / 24-11-2016 / 17:55:46 / cg"
!

msWindows8
    ^ #mswindows8
!

msWindowsVista
    ^ #mswindowsVista
!

msWindowsXP
    ^ #mswindowsXP
!

normal
    ^ #normal
! !

!ViewStyle methodsFor:'accessing'!

at:key
    ^ self at: key default: nil for: thisContext sender receiver

    "Modified: / 10-09-1995 / 10:59:38 / claus"
    "Modified: / 19-07-2016 / 21:41:21 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

at:key default:default
    ^ self at: key default: default for: thisContext sender receiver

    "Created: / 14-10-1997 / 00:21:15 / cg"
    "Modified: / 15-09-1998 / 21:47:13 / cg"
    "Modified: / 19-07-2016 / 21:41:03 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

at:key default:default for: classOrView
    "Retrieve a style resource (color, image, string...) for given key and
     view `class`. If not found, `default` is returned.

     Resource `key` is either a simple key (for example 'foo') or
     compound key (for example 'bar.foo'). Resource is looked up
     as follows:

     1. key '<class name>.foo' is looked up, if it exists
        then its value is returned.
     2. key 'bar.foo' is looked up, if it exists then
        then its value is returned (only if key is compound)
     3. key 'foo' is looked up,  if it exists then
        then its value is returned
     4. `default` value is returned.

    This has been added to support fine-grained resource (mainly color) specification
    allowing (easy) customization per widget class (in a somewhat predictable) way.
    All that while being backward compatible.
    "

    | class i key1 key2 key3 |

    class := classOrView class theNonMetaClass.

    "/ Update statistics if required
    usedKeys notNil ifTrue: [
        (usedKeys at: key ifAbsentPut:[WeakIdentitySet new]) add: class
    ].

    i := key indexOf: $..
    i ~~ 0 ifTrue:[ 
        key3 := (key copyFrom: i + 1).
        key2 := key.
    ] ifFalse:[ 
        key3 := key.
        key2 := nil.
    ].
    key1 := class class theNonMetaclass name , '.' , key3.

    (self includesKey:key1) ifTrue:[
        ^ (super at:key1 ifAbsent:default) value
    ].
    (key2 notNil and:[self includesKey:key2]) ifTrue:[
        ^ (super at:key2 ifAbsent:default) value
    ].
    (self includesKey:key3) ifTrue:[
        ^ (super at:key3 ifAbsent:default) value
    ].
    superPack notNil ifTrue: [
        ^ superPack at:key default:default for: classOrView.
    ].
    ^ default value.

    "Created: / 19-07-2016 / 22:21:04 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 15-09-2021 / 12:36:53 / Jan Vrany <jan.vrany@labware.com>"
!

colorAt: key
    ^ self colorAt: key default: nil for: thisContext sender receiver

    "Modified: / 19-07-2016 / 21:32:49 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

colorAt:key default:default
    ^ self colorAt:key default:default for: thisContext sender receiver

    "Modified (format): / 19-07-2016 / 21:36:15 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

colorAt:key default:default for:class
    "retrieve a color resource - also acquire a device color
     to avoid repeated color allocations later"

    |value device deviceColor|

    device := Display.

    value := self at:key default:default for: class.
    value isInteger ifTrue:[
        value := Color rgbValue:value
    ].
    (value notNil and:[device notNil]) ifTrue:[
        deviceColor := value onDevice:device.
        deviceColor notNil ifTrue:[^ deviceColor].
    ].
    ^ value

    "Created: / 19-07-2016 / 21:32:05 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 19-07-2016 / 22:40:21 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

doesNotUnderstand:aMessage
    ^ self at:(aMessage selector) default:nil
!

fontAt:key
   ^ self fontAt: key default: nil for: thisContext sender receiver.
!

fontAt:key default:default
    ^ self fontAt:key default:default for: thisContext sender receiver

    "Modified: / 19-07-2016 / 22:42:54 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

fontAt:key default:default for: class
    "retrieve a font resource - also acquire a device font
     to avoid repeated font allocations later"

    ^ self deviceResourceAt:key default:default for: class

    "Created: / 19-07-2016 / 22:42:19 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

is3D
    is3D isNil ifTrue:[
	is3D := self at:#is3D default:false.
    ].
    ^ is3D
!

name
    name isNil ifTrue:[
	name := self at:#name default:'noname'.
    ].
    ^ name
!

usedKeys
    ^ usedKeys
!

viewGrey
    viewGrey isNil ifTrue:[
        viewGrey := self at:#viewGrey default:nil.
    ].
    ^ viewGrey
! !

!ViewStyle methodsFor:'accessing-internals'!

rememberUsedKeys
    "start keeping a statistic on which keys are actually used and by who"
    
    usedKeys := Dictionary new.

    "Created: / 05-02-2021 / 21:38:51 / Jan Vrany <jan.vrany@labware.com>"
! !

!ViewStyle methodsFor:'error handling'!

nonexistingFileRead
    "here, a non-existing stylesheet is treated as an error"

    fileReadFailed := true.
    ^ self

    "Created: 6.9.1997 / 11:40:16 / cg"
! !

!ViewStyle methodsFor:'printing & storing'!

displayOn:aGCOrStream

    "/ what a kludge - Dolphin and Squeak mean: printOn: a stream;
    "/ ST/X (and some old ST80's) mean: draw-yourself on a GC.
    (aGCOrStream isStream) ifTrue:[
        aGCOrStream nextPutAll: self class name.
        (packsClassName notNil and:[projectPack notNil]) ifTrue: [
            aGCOrStream nextPutAll:' for: '.
            packsClassName printOn:aGCOrStream.
        ] ifFalse: [ 
            aGCOrStream nextPutAll:' fromFile: '.
            packsFileName asFilename asAbsoluteFilename pathName storeOn: aGCOrStream
        ].
        ^ self.
    ].

    ^ super displayOn:aGCOrStream

    "Created: / 21-09-2021 / 13:59:08 / Jan Vrany <jan.vrany@labware.com>"
! !

!ViewStyle methodsFor:'private'!

deviceResourceAt:key default:default for: class
    "retrieve a resource - also acquire a device version
     for the default display, to avoid repeated allocations later"

    |aResource deviceResource device|

    device := Display.

    aResource := self at:key default:default for: class.
    (aResource notNil and:[device notNil]) ifTrue:[
        deviceResource := aResource onDevice:device.
        deviceResource notNil ifTrue:[^ deviceResource].
    ].
    ^ aResource

    "Created: / 19-07-2016 / 22:41:17 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!ViewStyle methodsFor:'queries'!

isWindowsStyle
    "answer true if this is a MS-Windows style.
     XXX Currently we simply check the name"

    |nm|

    nm := name ? self name.

    ^ nm == #win95 
    or:[nm == #win98
    or:[nm == #winXP
    or:[nm == #winVista ]]]
!

isWindowsVistaStyle
    ^ (name ? self name) == #winVista

    "Modified (format): / 19-11-2016 / 15:49:48 / cg"
!

isWindowsXPStyle
    "answer true if this is a MS-Windows-XP (or later) style.
     XXX Currently we simply check the name"

    |nm|

    nm := name ? self name.

    ^ nm == #winXP 
    or:[nm == #winVista]
! !

!ViewStyle methodsFor:'special'!

newDerivedStyle
    "create and return a new style, which inherits from
     the receiver, but possibly overrides individual entries.
     This may be useful to give a single button some different
     look (in the future - read the comment in SimpleView>>viewStyle:)"

    ^ self class new
        superPack:self; 
        at:#is3D put:(self is3D);
        at:viewGrey put:(self viewGrey);
        yourself

    "
     |panel b1 b2 newStyle|

     panel := HorizontalPanelView new.
     panel add:(b1 := Button label:'oldStyle').
     panel add:(b2 := Button label:'newStyle').
     newStyle := b1 styleSheet newDerivedStyle.
     newStyle at:'button.activeBackgroundColor' put:Color blue.
     b2 styleSheet:newStyle.        

     panel open
    "
! !

!ViewStyle class methodsFor:'documentation'!

version
    ^ '$Header$'
!

version_CVS
    ^ '$Header$'
!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

